@@ -44,6 +44,9 @@
     ("utf" ("true" "false")))
   "Key=value options for arabverse environment.")
 
+(defvar LaTeX-arabluatex-color-options
+  '(()))
+
 (TeX-add-style-hook
  "arabluatex"
  (lambda ()
@@ -69,6 +72,9 @@
     '("arb" [ (TeX-arg-eval completing-read "Mode: "
 			    LaTeX-arabluatex-mode-options) ]
       t)
+    '("arbcolor" [ (TeX-arg-eval completing-read "Color: "
+			    LaTeX-arabluatex-color-options) ]
+      t)
     "SetHemistichDelim"
     '("bayt" 2)
     "abjad"
@@ -81,6 +87,8 @@
     "RLfootnote"
     "FixArbFtnmk"
     "MkArbBreak"
+    "arbmark"
+    '("newarbmark" 3)
     '("setRL" 0)
     '("setLR" 0))
    (LaTeX-add-environments
