@@ -119,6 +119,7 @@ sub glue_kern{
 	print  JPL "   (LABEL O 2)\n";
 	printf JPL "   (GLUE O 5 R %f R 0.0 R %f)\n", $half_width, $half_width;
 	printf JPL "   (GLUE O 6 R %f R 0.0 R %f)\n", $half_width, $half_width;
+	print  JPL "   (STOP)\n";
 	print  JPL "   (LABEL O 6)\n";
 	printf JPL "   (GLUE O 0 R %f R 0.0 R %f)\n", $half_width, $half_width;
 	printf JPL "   (GLUE O 1 R %f R 0.0 R %f)\n", $half_width, $half_width;
@@ -152,14 +153,24 @@ sub glue_kern{
 	print  JPL "   )\n";
 }
 sub chars_in_type_jis{
-	my ($type3add) = $ucs ? "U00B7" : "";
-	my ($type5) = $ucs ? "— ― … ‥" : "— … ‥";
+	my ($type1add, $type2add, $type3add, $type5, $type6add)=();
+	if ($ucs) {
+		$type1add = "UFF5F U3018 U3016 U301D U00AB U2329 U301A";
+		$type2add = "UFF60 U3019 U3017 U301F U00BB U232A U301B U301E";
+		$type3add = "U00B7";
+		$type5 = "— ― … ‥";
+		$type6add = "U203C U2047 U2048 U2049";
+	} else {
+		$type5 = "— … ‥";
+	}
 print JPL <<END_OF_DATA;
 (CHARSINTYPE O 1
    ‘ “ （ 〔 ［ ｛ 〈 《 「 『 【 
+   $type1add 
    )
 (CHARSINTYPE O 2
    、 ， ’ ” ） 〕 ］ ｝ 〉 》 」 』 】 
+   $type2add 
    )
 (CHARSINTYPE O 3
    ・ ： ； 
@@ -173,6 +184,7 @@ print JPL <<END_OF_DATA;
    )
 (CHARSINTYPE O 6
    ？ ！ 
+   $type6add 
    )
 END_OF_DATA
 }
@@ -222,7 +234,7 @@ sub write_char {
 		if   (&is_ucs_kigo){&print_kigo_char;}
 		elsif(&is_ucs_hira){&print_kana_char;}
 		elsif(&is_ucs_kata){&print_kana_char;}
-		elsif(&is_ucs_hankana && $direction eq "y"){&print_hankana_char;}
+#		elsif(&is_ucs_hankana && $hk_mode){&print_hankana_char;}
 		else{&print_char;}
 	}
     }
@@ -327,7 +339,7 @@ sub print_type_prop{
 
 sub get_charwidth{
 	my ($i,$dvicode)=@_;
-	my ($char,$eucchar,$testchar,$u,$l);
+	my ($char,$u,$l);
 
 	if (!$ucs) {
 		if ($dvicode>=0x2474 && $dvicode<=0x2476) {
