@@ -31,7 +31,8 @@ punctuationtr = {
 
 nulltr = {
    {a="%|", b=""},
-   {a="o[%S]-o", b=""},
+   {a="o", b=""},
+   {a="O[%S]-O", b=""},
    {a="[%^%_](.)", b="<??>%1"}
 }
 
@@ -119,13 +120,13 @@ lcuc = {
 hamzatrdmg = {
    -- next lines for ʾalif alone
    {a="(%.A)l%-(%^n)", b=".|l-%2"}, --additional (^n is lunar)
-   {a="([%(%[%|%<%s%-o])(%.A)l%-(%^n)", b="%1.|l-%3"}, --additional (^n is lunar) --p
+   {a="([%(%[%|%<%s%-O])(%.A)l%-(%^n)", b="%1.|l-%3"}, --additional (^n is lunar) --p
    {a="(%.A)l%-([%_%^%.]?[tdrzsnc])", b=".|%2-%2"},
-   {a="([%(%[%|%<%s%-o])(%.A)l%-([%_%^%.]?[tdrzsnc])", b="%1.|%3-%3"}, --p
+   {a="([%(%[%|%<%s%-O])(%.A)l%-([%_%^%.]?[tdrzsnc])", b="%1.|%3-%3"}, --p
    {a="(%.A)([uai])l%-(%^n)", b="||%2l-%3"}, --additional (^n is lunar)
-   {a="([%(%[%|%<%s%-o])(%.A)([uai])l%-(%^n)", b="%1||%3l-%4"}, --additional (^n is lunar) --p
+   {a="([%(%[%|%<%s%-O])(%.A)([uai])l%-(%^n)", b="%1||%3l-%4"}, --additional (^n is lunar) --p
    {a="(%.A)([uai])l%-([%_%^%.]?[tdrzsnc])", b="||%2%3-%3"},
-   {a="([%(%[%|%<%s%-o])(%.A)([uai])l%-([%_%^%.]?[tdrzsnc])", b="%1||%3%4-%4"}, --p
+   {a="([%(%[%|%<%s%-O])(%.A)([uai])l%-([%_%^%.]?[tdrzsnc])", b="%1||%3%4-%4"}, --p
    {a="(%.A)([^uai])", b=".|%2"},
    {a="(%.A)([uai])", b="||%2"},
    -- hard coded hamza
@@ -158,10 +159,10 @@ hamzatrdmg = {
    {a="(A)(')", b="%1ʾ"}, -- historic madda
    -- initial (needs both ^ and %W patterns)
    -- 'aw: the diphthong is to be resolved into 'awi' (next 8 lines)
-   {a="^('aw)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1i"},
-   {a="(%W)('aw)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1%2i"},
-   {a="^('aw)(o)(\"?[uai])([%S]-o)", b="%1i"},
-   {a="(%W)('aw)(o)(\"?[uai])([%S]-o)", b="%1%2i"},
+   {a="^('aw)(O)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-O)", b="%1i"},
+   {a="(%W)('aw)(O)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-O)", b="%1%2i"},
+   {a="^('aw)(O)(\"?[uai])([%S]-O)", b="%1i"},
+   {a="(%W)('aw)(O)(\"?[uai])([%S]-O)", b="%1%2i"},
    {a="^('aw)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1i%2%3"},
    {a="(%W)('aw)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1%2i%3%4"},
    {a="^('aw)(%s)([%(%[%|%<]?\"?[uai])", b="%1i%2%3"}, --p
@@ -202,22 +203,22 @@ hamzatrdmg = {
 }
 
 tanwintrdmg = {
-   {a="%-?([uai]NU)(o)([ui])([%S]-o)", b="\\arbup{un%3}"},
+   {a="%-?([uai]NU)(O)([ui])([%S]-O)", b="\\arbup{un%3}"},
    {a="%-?([uai]NU)(%s)([ui])", b="\\arbup{un%3}%2'"},
-   {a="%-?(iNI)(o)([ui])([%S]-o)", b="i\\arbup{n%3}"},
+   {a="%-?(iNI)(O)([ui])([%S]-O)", b="i\\arbup{n%3}"},
    {a="%-?(iNI)(%s)([ui])", b="i\\arbup{n%3}%2'"},
-   {a="(o[%S]-)([uai]N[UI])(o)(\"?[ui])", b="'"},
+   {a="(O[%S]-)([uai]N[UI])(O)(\"?[ui])", b="'"},
    {a="%-?uNU", b="\\arbup{un}"},
    {a="%-?aNU", b="\\arbup{an}"},
    {a="%-?iNU", b="\\arbup{in}"},
    {a="%-?iNI", b="i\\arbup{n}"},
    -- tanwīn preceding ʾalif conjunctionis
-   {a="%-?(uN)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="\\arbup{uni}"},
-   {a="%-?(aN)(_A)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="ạ\\arbup{ni}"},
-   {a="%-?(aN)(Y)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="ạ\\arbup{ni}"},
-   {a="(T)%-?(aN)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="t\\arbup{ani}"},
-   {a="([^TA])%-?(aN)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1\\arbup{ani}"},
-   {a="%-?(iN)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="\\arbup{ini}"},
+   {a="%-?(uN)(O)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-O)", b="\\arbup{uni}"},
+   {a="%-?(aN)(_A)(O)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-O)", b="ạ\\arbup{ni}"},
+   {a="%-?(aN)(Y)(O)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-O)", b="ạ\\arbup{ni}"},
+   {a="(T)%-?(aN)(O)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-O)", b="t\\arbup{ani}"},
+   {a="([^TA])%-?(aN)(O)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-O)", b="%1\\arbup{ani}"},
+   {a="%-?(iN)(O)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-O)", b="\\arbup{ini}"},
    {a="%-?(uN)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="\\arbup{uni}%2%3"},
    {a="%-?(aN)(_A)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="ạ\\arbup{ni}%3%4"},
    {a="%-?(aN)(Y)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="ạ\\arbup{ni}%3%4"},
@@ -225,12 +226,12 @@ tanwintrdmg = {
    {a="([^TA])%-?(aN)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1\\arbup{ani}%3%4"},
    {a="%-?(iN)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="\\arbup{ini}%2%3"},
    -- tanwīn preceding 'lla_dI/'llatI
-   {a="%-?(uN)(o)('lla[%_]?[dt])([%S]-o)", b="\\arbup{uni}"},
-   {a="%-?(aN)(_A)(o)('lla[%_]?[dt])([%S]-o)", b="ạ\\arbup{ni}"},
-   {a="%-?(aN)(Y)(o)('lla[%_]?[dt])([%S]-o)", b="ạ\\arbup{ni}"},
-   {a="(T)%-?(aN)(o)('lla[%_]?[dt])([%S]-o)", b="t\\arbup{ani}"},
-   {a="([^TA])%-?(aN)(o)('lla[%_]?[dt])([%S]-o)", b="%1\\arbup{ani}"},
-   {a="%-?(iN)(o)('lla[%_]?[dt])([%S]-o)", b="\\arbup{ini}"},
+   {a="%-?(uN)(O)('lla[%_]?[dt])([%S]-O)", b="\\arbup{uni}"},
+   {a="%-?(aN)(_A)(O)('lla[%_]?[dt])([%S]-O)", b="ạ\\arbup{ni}"},
+   {a="%-?(aN)(Y)(O)('lla[%_]?[dt])([%S]-O)", b="ạ\\arbup{ni}"},
+   {a="(T)%-?(aN)(O)('lla[%_]?[dt])([%S]-O)", b="t\\arbup{ani}"},
+   {a="([^TA])%-?(aN)(O)('lla[%_]?[dt])([%S]-O)", b="%1\\arbup{ani}"},
+   {a="%-?(iN)(O)('lla[%_]?[dt])([%S]-O)", b="\\arbup{ini}"},
    {a="%-?(uN)(%s)('lla[%_]?[dt])", b="\\arbup{uni}%2%3"},
    {a="%-?(aN)(_A)(%s)('lla[%_]?[dt])", b="ạ\\arbup{ni}%3%4"},
    {a="%-?(aN)(Y)(%s)('lla[%_]?[dt])", b="ạ\\arbup{ni}%3%4"},
@@ -238,13 +239,13 @@ tanwintrdmg = {
    {a="([^TA])%-?(aN)(%s)('lla[%_]?[dt])", b="%1\\arbup{ani}%3%4"},
    {a="%-?(iN)(%s)('lla[%_]?[dt])", b="\\arbup{ini}%2%3"},
    -- tanwīn + alif without hamza and kasra (ibn) or dhamma (uhrub)
-   {a="%-?(uN)(o)([ui])([%S]-o)", b="\\arbup{un%3}"},
-   {a="%-?(aN)(_A)(o)([ui])([%S]-o)", b="ạ\\arbup{n%4}"},
-   {a="%-?(aN)(Y)(o)([ui])([%S]-o)", b="ạ\\arbup{n%4}"},
-   {a="(T)%-?(aN)(o)([ui])([%S]-o)", b="t\\arbup{an%4}"},
-   {a="([^TA])%-?(aN)(o)([ui])([%S]-o)", b="%1\\arbup{an%4}"},
-   {a="%-?(iN)(o)([ui])([%S]-o)", b="\\arbup{in%3}"},
-   {a="(o[%S]-)([uai]N)(o)(\"?[ui])", b="'"},
+   {a="%-?(uN)(O)([ui])([%S]-O)", b="\\arbup{un%3}"},
+   {a="%-?(aN)(_A)(O)([ui])([%S]-O)", b="ạ\\arbup{n%4}"},
+   {a="%-?(aN)(Y)(O)([ui])([%S]-O)", b="ạ\\arbup{n%4}"},
+   {a="(T)%-?(aN)(O)([ui])([%S]-O)", b="t\\arbup{an%4}"},
+   {a="([^TA])%-?(aN)(O)([ui])([%S]-O)", b="%1\\arbup{an%4}"},
+   {a="%-?(iN)(O)([ui])([%S]-O)", b="\\arbup{in%3}"},
+   {a="(O[%S]-)([uai]N)(O)(\"?[ui])", b="'"},
    {a="%-?(uN)(%s)([ui])", b="\\arbup{un%3}%2'"},
    {a="%-?(aN)(_A)(%s)([ui])", b="ạ\\arbup{n%4}%3'"},
    {a="%-?(aN)(Y)(%s)([ui])", b="ạ\\arbup{n%4}%3'"},
@@ -254,6 +255,12 @@ tanwintrdmg = {
    --
 -- {a="uN", b="\\arbup{un}"}, (now included in the last line of this table)
    {a="%-?(\"?At)%-?([ui])N", b="\\arbup{%1%2n}"},
+   -- needed by \arbcolor:
+   {a="%-?(aN)(O[%S]-%_AO)", b="ạ\\arbup{n}"},
+   {a="%-?(aN)(O[%S]-YO)", b="ạ\\arbup{n}"},
+   {a="(O[%S]-TO)%-?(\"?aN)", b="\\arbup{an}"},
+   {a="(O[%S]-[^TA]O)%-?(\"?aN)", b="\\arbup{an}"},
+   --
    {a="%-?(aN)(_A)", b="ạ\\arbup{n}"},
    {a="%-?(aN)(Y)", b="ạ\\arbup{n}"},
    {a="(T)%-?(\"?aN)", b="t\\arbup{an}"},
@@ -266,38 +273,38 @@ trigraphstrdmg = { -- trigraphs or more
    {a="^'ll(a)([%_]?[dt])", b="'ll%1%2"},
    {a="([%(%[%|%<%s])'ll(a)([%_]?[dt])", b="%1'll%2%3"}, --p
    -- law: the diphthong is to be resolved into 'awi' (next 8 lines)
-   {a="^(law)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1i"},
-   {a="(%W)(law)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1%2i"},
-   {a="^(law)(o)(\"?[uai])([%S]-o)", b="%1i"},
-   {a="(%W)(law)(o)(\"?[uai])([%S]-o)", b="%1%2i"},
+   {a="^(law)(O)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-O)", b="%1i"},
+   {a="(%W)(law)(O)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-O)", b="%1%2i"},
+   {a="^(law)(O)(\"?[uai])([%S]-O)", b="%1i"},
+   {a="(%W)(law)(O)(\"?[uai])([%S]-O)", b="%1%2i"},
    {a="^(law)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1i%2%3"},
    {a="(%W)(law)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1%2i%3%4"},
    {a="^(law)(%s)([%(%[%|%<]?\"?[uai])", b="%1i%2%3"}, --p
    {a="(%W)(law)(%s)([%(%[%|%<]?\"?[uai])", b="%1%2i%3%4"}, --p
    -- al- + lām
    {a="^(a)l%-(l)", b="%1l-%2"},
-   {a="([%(%[%|%<%s%-o])(a)l%-(l)", b="%1%2l-%3"}, --p
+   {a="([%(%[%|%<%s%-O])(a)l%-(l)", b="%1%2l-%3"}, --p
    -- al- + solar consonant ('c' and '^n' are additional characters)
    {a="^(a)l%-(%^n)", b="%1l-%2"}, -- ^n is lunar
-   {a="([%(%[%|%<%s%-o])(a)l%-(%^n)", b="%1%2l-%3"}, --^n is lunar --p
+   {a="([%(%[%|%<%s%-O])(a)l%-(%^n)", b="%1%2l-%3"}, --^n is lunar --p
    {a="^(a)l%-([%_%^%.]?[tdrzsnc])", b="%1%2-%2"},
-   {a="([%(%[%|%<%s%-o])(a)l%-([%_%^%.]?[tdrzsnc])", b="%1%2%3-%3"}, --p
+   {a="([%(%[%|%<%s%-O])(a)l%-([%_%^%.]?[tdrzsnc])", b="%1%2%3-%3"}, --p
    -- assim. art. + solar consonant ('c' and '^n' are additional characters)
    {a="^(a)(%^n)%-", b="%1l-"}, -- ^n is lunar
-   {a="([%(%[%|%<%s%-o])(a)(%^n)%-", b="%1%2l-"}, --^n is lunar --p
+   {a="([%(%[%|%<%s%-O])(a)(%^n)%-", b="%1%2l-"}, --^n is lunar --p
    {a="^(a)([%_%^%.]?[tdrzsnc])%-", b="%1%2-"},
-   {a="([%(%[%|%<%s%-o])(a)([%_%^%.]?[tdrzsnc])%-", b="%1%2%3-"}, --p
+   {a="([%(%[%|%<%s%-O])(a)([%_%^%.]?[tdrzsnc])%-", b="%1%2%3-"}, --p
    -- al- + initial unstable hamza
    {a="^(a)l%-(\"?[uai])", b="%1l-%2"},
-   {a="([%(%[%|%<%s%-o])(a)l%-(\"?[uai])", b="%1%2l-%3"}, --p
+   {a="([%(%[%|%<%s%-O])(a)l%-(\"?[uai])", b="%1%2l-%3"}, --p
    -- li-/la- + art. + initial unstable hamza is a special orthography
    {a="l([ai])%-l%-([uai])", b="l%1-l-%2"},
    -- al- + lunar consonant (i.e. what remains)
    {a="^(a)l%-", b="%1l-"},
-   {a="([%(%[%|%<%s%-o])(a)l%-", b="%1%2l-"}, --p
+   {a="([%(%[%|%<%s%-O])(a)l%-", b="%1%2l-"}, --p
    -- diphthongs to be resolved before ʾalif conjunctionis
-   {a="(aw)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1u"},
-   {a="(ay)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1i"},
+   {a="(aw)(O)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-O)", b="%1u"},
+   {a="(ay)(O)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-O)", b="%1i"},
    {a="(aw)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1u%2%3"},
    {a="(ay)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1i%2%3"},
    -- art. with waṣla + lām
@@ -340,7 +347,7 @@ trigraphstrdmg = { -- trigraphs or more
 idghamtrdmg = {
    -- assimilations
    {a="(n)(}?)(%s)([rlmnwy])", b="%4%2%3%4"},
-   {a="(n)(}?)(o)([rlmnwy])([%S]-o)", b="%4%2"}
+   {a="(n)(}?)(O)([rlmnwy])([%S]-O)", b="%4%2"}
 }
 
 digraphstrdmg = {
@@ -354,19 +361,20 @@ digraphstrdmg = {
    {a="(%W)(\"?[uai])", b="%1%2"},      -- initial alif without hamza
    -- this is not necessary, take out for now:
 -- {a="([%_]?[uaiUAIY])(%s)([uai])", b="%1%2'"}, -- initial alif without hamza
-   {a="(aw)(o)(\"?[uai])([%S]-o)", b="%1u"},
+   {a="(aw)(O)(\"?[uai])([%S]-O)", b="%1u"},
    {a="(aw)(%s)([%(%[%|%<]?)(\"?[uai])", b="%1u%2%3'"}, --p
-   {a="(ay)(o)(\"?[uai])([%S]-o)", b="%1i"},
+   {a="(ay)(O)(\"?[uai])([%S]-O)", b="%1i"},
    {a="(ay)(%s)([%(%[%|%<]?)(\"?[uai])", b="%1i%2%3'"}, --p
-   {a="(aW)(o)(\"?[uai])([%S]-o)", b="awu"},
-   {a="(UA)(o)(\"?[uai])([%S]-o)", b="u"},
-   {a="(%_A)(o)(\"?[uai])([%S]-o)", b="ạ"},
-   {a="(Y)(o)(\"?[uai])([%S]-o)", b="ạ"},
-   {a="(%_a)(o)(\"?[uai])([%S]-o)", b="a"},
-   {a="(A)(o)(\"?[uai])([%S]-o)", b="a"},
-   {a="([%_]?[Uu])(o)(\"?[uai])([%S]-o)", b="u"},
-   {a="([%_]?[Ii])(o)(\"?[uai])([%S]-o)", b="i"},
-   {a="(o[%S]-)([UAIYWuaiyw])(o)(\"?[uai])", b="'"},
+   {a="(aW)(O)(\"?[uai])([%S]-O)", b="awu"},
+   {a="(UA)(O)(\"?[uai])([%S]-O)", b="u"},
+   {a="(%_A)(O)(\"?[uai])([%S]-O)", b="ạ"},
+   {a="(Y)(O)(\"?[uai])([%S]-O)", b="ạ"},
+   {a="(%_a)(O)(\"?[uai])([%S]-O)", b="a"},
+   {a="(A)(O)(\"?[uai])([%S]-O)", b="a"},
+   {a="([%_]?[Uu])(O)(\"?[uai])([%S]-O)", b="u"},
+   {a="([%_]?[Ii])(O)(\"?[uai])([%S]-O)", b="i"},
+   {a="(O[%S]-)([%'a]l%-)(O)(\"?[uai])", b="'"},
+   {a="(O[%S]-)([UAIYWuaiyw])(O)(\"?[uai])", b="'"},
    {a="(aW)(%s)([%(%[%|%<]?)(\"?[uai])", b="awu%2%3%4"}, --p
    {a="(UA)(%s)([%(%[%|%<]?)(\"?[uai])", b="u%2%3'"}, --p
    {a="([^%_][uai])(%s)([%(%[%|%<]?)(\"?[uai])", b="%1%2%3'"}, --p
@@ -377,26 +385,26 @@ digraphstrdmg = {
    {a="([%_]?[Uu])(%s)([%(%[%|%<]?)(\"?[uai])", b="u%2%3'"}, --p
    {a="([%_]?[Ii])(%s)([%(%[%|%<]?)(\"?[uai])", b="i%2%3'"}, --p
    -- ʾiʿrāb hyphen (begin)
-   {a="(%-)(\"?[UI]na)(%p?%s)", b="\\arbup{%2}%3"},
-   {a="(%-)(\"?[UI]na)(%p?)$", b="\\arbup{%2}%3"},
-   {a="(%-)(\"?At[ui])(%p?%s)", b="\\arbup{%2}%3"},
-   {a="(%-)(\"?At[ui])(%p?)$", b="\\arbup{%2}%3"},
-   {a="(%-)(\"?Ani)(%p?%s)", b="\\arbup{%2}%3"},
-   {a="(%-)(\"?Ani)(%p?)$", b="\\arbup{%2}%3"},
-   {a="(%-)(\"?ayni)(%p?%s)", b="\\arbup{%2}%3"},
-   {a="(%-)(\"?ayni)(%p?)$", b="\\arbup{%2}%3"},
-   {a="(%-)(\"?[uai])(%p?%s)", b="\\arbup{%2}%3"},
-   {a="(%-)(\"?[uai])(%p?)$", b="\\arbup{%2}%3"},
+   {a="(%-)(\"?[UI]na)(%p*%s)", b="\\arbup{%2}%3"},
+   {a="(%-)(\"?[UI]na)(%p*)$", b="\\arbup{%2}%3"},
+   {a="(%-)(\"?At%.?[ui])(%p*%s)", b="\\arbup{%2}%3"},
+   {a="(%-)(\"?At%.?[ui])(%p*)$", b="\\arbup{%2}%3"},
+   {a="(%-)(\"?Ani)(%p*%s)", b="\\arbup{%2}%3"},
+   {a="(%-)(\"?Ani)(%p*)$", b="\\arbup{%2}%3"},
+   {a="(%-)(\"?%.?ayni)(%p*%s)", b="\\arbup{%2}%3"},
+   {a="(%-)(\"?%.?ayni)(%p*)$", b="\\arbup{%2}%3"},
+   {a="(%-)(\"?%.?[uai])(%p*%s)", b="\\arbup{%2}%3"},
+   {a="(%-)(\"?%.?[uai])(%p*)$", b="\\arbup{%2}%3"},
    -- ʾiʿrāb hyphen (end) shorten long vowels preceding ʾalif
    -- conjunctionis—without forgetting 'lla_dI
    {a="(U)(A)", b="U"},
-   {a="(aW)(o)('[%_%^%.]?[l'btjghxdrzs`fqkmnwy][%-l])([%S]-o)", b="awu"},
-   {a="(%_a)(o)('[%_%^%.]?[l'btjghxdrzs`fqkmnwy][%-l])([%S]-o)", b="a"},
-   {a="(%_A)(o)('[%_%^%.]?[l'btjghxdrzs`fqkmnwy][%-l])([%S]-o)", b="ạ"},
-   {a="(A)(o)('[%_%^%.]?[l'btjghxdrzs`fqkmnwy][%-l])([%S]-o)", b="a"},
-   {a="(Y)(o)('[%_%^%.]?[l'btjghxdrzs`fqkmnwy][%-l])([%S]-o)", b="ạ"},
-   {a="([%_]?[Uu])(o)('[%_%^%.]?[l'btjghxdrzs`fqkmnwy][%-l])([%S]-o)", b="u"},
-   {a="([%_]?[Ii])(o)('[%_%^%.]?[l'btjghxdrzs`fqkmnwy][%-l])([%S]-o)", b="i"},
+   {a="(aW)(O)('[%_%^%.]?[l'btjghxdrzs`fqkmnwy][%-l])([%S]-O)", b="awu"},
+   {a="(%_a)(O)('[%_%^%.]?[l'btjghxdrzs`fqkmnwy][%-l])([%S]-O)", b="a"},
+   {a="(%_A)(O)('[%_%^%.]?[l'btjghxdrzs`fqkmnwy][%-l])([%S]-O)", b="ạ"},
+   {a="(A)(O)('[%_%^%.]?[l'btjghxdrzs`fqkmnwy][%-l])([%S]-O)", b="a"},
+   {a="(Y)(O)('[%_%^%.]?[l'btjghxdrzs`fqkmnwy][%-l])([%S]-O)", b="ạ"},
+   {a="([%_]?[Uu])(O)('[%_%^%.]?[l'btjghxdrzs`fqkmnwy][%-l])([%S]-O)", b="u"},
+   {a="([%_]?[Ii])(O)('[%_%^%.]?[l'btjghxdrzs`fqkmnwy][%-l])([%S]-O)", b="i"},
    --p (next 7 lines, just after %s)
    {a="(aW)(%s)([%(%[%|%<]?['][%_%^%.]?[l'btjghxdrzs`fqkmnwy][%-l])", b="awu%2%3"},
    {a="(%_a)(%s)([%(%[%|%<]?['][%_%^%.]?[l'btjghxdrzs`fqkmnwy][%-l])", b="a%2%3"},
@@ -406,18 +414,18 @@ digraphstrdmg = {
    {a="([%_]?[Uu])(%s)([%(%[%|%<]?['][%_%^%.]?[l'btjghxdrzs`fqkmnwy][%-l])", b="u%2%3"},
    {a="([%_]?[Ii])(%s)([%(%[%|%<]?['][%_%^%.]?[l'btjghxdrzs`fqkmnwy][%-l])", b="i%2%3"},
    {a="%-%-", b=""},
-   {a="iyyaT(%p?)$", b="īyaT%1"},
-   {a="iyyaT(%p?%s)", b="īyaT%1"},
-   {a="iyy(%p?)$", b="ī%1"},
-   {a="iyy(%p?%s)", b="ī%1"},
+   {a="iyyaT(%p*)$", b="īyaT%1"},
+   {a="iyyaT(%p*%s)", b="īyaT%1"},
+   {a="iyy(%p*)$", b="ī%1"},
+   {a="iyy(%p*%s)", b="ī%1"},
    --   {a="T([^uai])", b="%1"},
    {a="T(\\arbup)", b="t%1"},
    {a="([a%'][%_%^%.]?[tdrzsln]%-)(%S-)T([%(%[%|%<%s])(a[%_%^%.]?[tdrzsln]%-)", b="%1%2h%3%4"}, --p
    {a="T([%(%[%|%<%s])(a[%_%^%.]?[tdrzsln]%-)", b="t%1%2"}, --p
    {a="T([%|\"])", b="t%1"},
-   {a="T(%p?%s)", b="h%1"},
-   {a="T(%p?)$", b="h%1"},
-   {a="T(%p?)(%W)", b="h%1%2"},
+   {a="T(%p*%s)", b="h%1"},
+   {a="T(%p*)$", b="h%1"},
+   {a="T(%p*)(%W)", b="h%1%2"},
    {a="_t", b="ṯ"},
    {a="%^g", b="ǧ"},
    {a="%.h", b="ḥ"},
@@ -492,6 +500,9 @@ longvtrdmg = {
 shortvtrdmg = {
    {a="([uai])([uai])([uai])", b="/%1,%2,%3/"},
    {a="([uai])([uai])", b="/%1,%2/"},
+   {a="%.u", b="u"},
+   {a="%.a", b="a"},
+   {a="%.i", b="i"},
    {a="u", b="u"},
    {a="a", b="a"},
    {a="i", b="i"}
@@ -581,6 +592,12 @@ tanwintrloc = {
    {a="%-?iNI", b="in"},
    {a="%-?(\"?At)%-?([ui])N", b="%1%2n"},
    {a="%-?([ui])N", b="%1n"},
+   -- needed by \arbcolor:
+   {a="%-?(aN)(O[%S]-%_AO)", b="an"},
+   {a="%-?(aN)(O[%S]-YO)", b="an"},
+   {a="(O[%S]-TO)%-?(\"?aN)", b="an"},
+   {a="(O[%S]-[^TA]O)%-?(\"?aN)", b="an"},
+   --
    {a="%-?(aN)(_A)", b="an"},
    {a="%-?(aN)(Y)", b="an"},
    {a="(T)%-?(\"?aN)", b="tan"},
@@ -651,32 +668,32 @@ trigraphstrloc = { -- trigraphs or more
 
 digraphstrloc = {
    -- discard the ʾiʿrāb hyphen (begin)
-   {a="(%-)(\"?[UI]na)(%p?%s)", b="%2%3"},
-   {a="(%-)(\"?[UI]na)(%p?)$", b="%2%3"},
-   {a="(%-)(\"?At[ui])(%p?%s)", b="%2%3"},
-   {a="(%-)(\"?At[ui])(%p?)$", b="%2%3"},
-   {a="(%-)(\"?Ani)(%p?%s)", b="%2%3"},
-   {a="(%-)(\"?Ani)(%p?)$", b="%2%3"},
-   {a="(%-)(\"?ayni)(%p?%s)", b="%2%3"},
-   {a="(%-)(\"?ayni)(%p?)$", b="%2%3"},
-   {a="(%-)([uai])(%p?%s)", b="%2%3"},
-   {a="(%-)([uai])(%p?)$", b="%2%3"},
+   {a="(%-)(\"?[UI]na)(%p*%s)", b="%2%3"},
+   {a="(%-)(\"?[UI]na)(%p*)$", b="%2%3"},
+   {a="(%-)(\"?At[ui])(%p*%s)", b="%2%3"},
+   {a="(%-)(\"?At[ui])(%p*)$", b="%2%3"},
+   {a="(%-)(\"?Ani)(%p*%s)", b="%2%3"},
+   {a="(%-)(\"?Ani)(%p*)$", b="%2%3"},
+   {a="(%-)(\"?ayni)(%p*%s)", b="%2%3"},
+   {a="(%-)(\"?ayni)(%p*)$", b="%2%3"},
+   {a="(%-)([uai])(%p*%s)", b="%2%3"},
+   {a="(%-)([uai])(%p*)$", b="%2%3"},
    -- discard the ʾiʿrāb hyphen (end)
    {a="(%-)(\"?[uai])", b="%1%2"}, -- hyphen + initial alif without hamza
    {a="^(\"?[uai])", b="%1"},      -- initial alif without hamza
    {a="(%s)([uai])", b="%1%2"}, -- initial alif without hamza
    {a="%-%-", b=""},
    {a="uww", b="ūw"},
-   {a="iyy(%p?)$", b="ī%1"},
-   {a="iyy(%p?%s)", b="ī%1"},
+   {a="iyy(%p*)$", b="ī%1"},
+   {a="iyy(%p*%s)", b="ī%1"},
    {a="iyy", b="īy"},
    {a="([tkdsg])(h)", b="%1'%2"},
    --   {a="T([^uai])", b="h%1"},
    {a="([a%']l%-)(%S-)T([%(%[%|%<%s])(al%-)", b="%1%2h%3%4"}, --p
    {a="T([%(%[%|%<%s])(al%-)", b="t%1%2"}, --p
    {a="T([%|\"])", b="t%1"},
-   {a="T(%p?)$", b="h%1"},
-   {a="T(%p?%s)", b="h%1"},
+   {a="T(%p*)$", b="h%1"},
+   {a="T(%p*%s)", b="h%1"},
    {a="_t", b="th"},
    {a="%^g", b="j"},
    {a="%.h", b="ḥ"},
@@ -750,6 +767,9 @@ longvtrloc = {
 shortvtrloc = {
    {a="([uai])([uai])([uai])", b="/%1,%2,%3/"},
    {a="([uai])([uai])", b="/%1,%2/"},
+   {a="%.u", b="u"},
+   {a="%.a", b="a"},
+   {a="%.i", b="i"},
    {a="u", b="u"},
    {a="a", b="a"},
    {a="i", b="i"}
@@ -902,31 +922,32 @@ trigraphstrarabica = { -- trigraphs or more
 digraphstrarabica = {
    {a="([uai]%-)(\"?[uai])", b="%1"}, -- hyphen + initial alif without hamza
    {a="([UAIYuai])(%s)([%(%[%|%<]?)(\"?[uai])", b="%1%2%3"}, --p
+   {a="(O[%S]-)([UAIuai])(O)(\"?[uai])", b=""},
    {a="@", b=""}, -- remove the tag before the former hamza
    -- discard the ʾiʿrāb hyphen (begin)
-   {a="(%-)(\"?[UI]na)(%p?%s)", b="%2%3"},
-   {a="(%-)(\"?[UI]na)(%p?)$", b="%2%3"},
-   {a="(%-)(\"?At[ui])(%p?%s)", b="%2%3"},
-   {a="(%-)(\"?At[ui])(%p?)$", b="%2%3"},
-   {a="(%-)(\"?Ani)(%p?%s)", b="%2%3"},
-   {a="(%-)(\"?Ani)(%p?)$", b="%2%3"},
-   {a="(%-)(\"?ayni)(%p?%s)", b="%2%3"},
-   {a="(%-)(\"?ayni)(%p?)$", b="%2%3"},
-   {a="(%-)([uai])(%p?%s)", b="%2%3"},
-   {a="(%-)([uai])(%p?)$", b="%2%3"},
+   {a="(%-)(\"?[UI]na)(%p*%s)", b="%2%3"},
+   {a="(%-)(\"?[UI]na)(%p*)$", b="%2%3"},
+   {a="(%-)(\"?At[ui])(%p*%s)", b="%2%3"},
+   {a="(%-)(\"?At[ui])(%p*)$", b="%2%3"},
+   {a="(%-)(\"?Ani)(%p*%s)", b="%2%3"},
+   {a="(%-)(\"?Ani)(%p*)$", b="%2%3"},
+   {a="(%-)(\"?ayni)(%p*%s)", b="%2%3"},
+   {a="(%-)(\"?ayni)(%p*)$", b="%2%3"},
+   {a="(%-)([uai])(%p*%s)", b="%2%3"},
+   {a="(%-)([uai])(%p*)$", b="%2%3"},
    -- discard the ʾiʿrāb hyphen (end)
    {a="(%-)(\"?[uai])", b="%1%2"}, -- hyphen + initial alif without hamza
    {a="^(\"?[uai])", b="%1"},      -- initial alif without hamza
    {a="(%s)([uai])", b="%1%2"}, -- initial alif without hamza
    {a="%-%-", b=""},
-   {a="iyy(%p?)$", b="ī%1"},
-   {a="iyy(%p?%s)", b="ī%1"},
+   {a="iyy(%p*)$", b="ī%1"},
+   {a="iyy(%p*%s)", b="ī%1"},
    --   {a="T([^uai])", b="h%1"},
    {a="([a%']l%-)(%S-)aT([%(%[%|%<%s])(al%-)", b="%1%2a%3%4"}, --p
    {a="aT([%(%[%|%<%s])(al%-)", b="at%1%2"}, --p
    {a="T([%|\"])", b="t%1"},
-   {a="aT(%p?)$", b="a%1"},
-   {a="aT(%p?%s)", b="a%1"},
+   {a="aT(%p*)$", b="a%1"},
+   {a="aT(%p*%s)", b="a%1"},
    {a="_t", b="ṯ"},
    {a="%^g", b="ǧ"},
    {a="%.h", b="ḥ"},
